# Amplitude-fit.py.     I. Marti-Vidal.      IMPRS Blackboard Lectures 2011.
# WHAT WOULD HAPPEN IF YOU LOOSE THE PHASES OF ALL YOUR VISIBILITIES? OH OH OOOH

import numpy as np
import pylab as pl
import scipy.special as sp

# Parameters:
sigma = 1.0     # Thermal noise of visibilities.
flux = 0.5      # Flux density of observed (compact) source.
nmonte = 10000  # Number of visibilities to simulate.
bins = 30       # Number of bins in histogram plots.

# Simulate visibilities:
real = np.random.normal(flux,sigma,nmonte)
imag = np.random.normal(0.0,sigma,nmonte)
amplit = np.sqrt(real*real+imag*imag)

# Compute the amplitude average and compare to true real source flux (since 
# the noise does NOT average to 0, this doesn't seem a good way to preoceed!):
print "\n True flux (Jy):         ",flux
print   " Average amplitude (Jy): ",np.average(amplit)," +/- ",np.std(amplit)
#holf=raw_input("\n Holding. Press ENTER to continue") 

# Estimate flux from MLE (WARNING: Amplitude noise is NOT Gaussian. A simple 
# least-squares fit will NOT work!).:
fluxes = np.linspace(0,1.5*flux,100)
MLE = [np.sum( fluxes[i]**2./(2.*sigma**2.) - np.log(sp.i0(amplit*fluxes[i]/sigma**2.))) for i in range(len(fluxes))]

# Some plots:
pl.hist(real,bins); pl.title("REAL PART OF VISIBILITES")
pl.xlabel("Real part (Jy)")
pl.figure()
pl.hist(imag,bins); pl.title("IMAGINARY PART OF VISIBILITES")
pl.xlabel("Imaginary part (Jy)")
pl.figure()
pl.hist(amplit,bins); pl.title("AMPLITUDE OF VISIBILITES")
pl.xlabel("Amplitude (Jy)")
pl.figure()
pl.plot(amplit,'.'); pl.title("WHAT'S YOUR GUESS OF THE SOURCE FLUX DENSITY?")
pl.xlabel("Sample #")
pl.ylabel("Amplitude (Jy)")
#pl.show()

#holf=raw_input("\n Holding. Press ENTER to continue")
# Maximum likelihood!
pl.figure()
pl.plot(fluxes,MLE,'.'); pl.title("(MINUS) LIKELIHOOD OF FLUX DENSITY")
pl.xlabel("Flux density (Jy)")
pl.ylabel("-log(P) + K")

pl.show()
